from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.LomObject import LomObject
    from Live.Song import Song



class TuningSystem(LomObject):
    """Represents a Tuning System and its properties."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_highest_note_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "highest_note" has changed.
        """
        ...

    def add_lowest_note_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "lowest_note" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_note_tunings_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "note_tunings" has changed.
        """
        ...

    def add_reference_pitch_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "reference_pitch" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Song:
        """Get the canonical parent of the TuningSystem."""
        ...

    @property
    def highest_note(self) -> PitchClassAndOctave:
        """
        Get/Set the highest note of the current tuning system, where the first entry is
        the index within the pseudo octave and the second entry is the octave.
        """
        ...

    @highest_note.setter
    def highest_note(self, value: PitchClassAndOctave) -> None: ...

    def highest_note_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "highest_note".
        """
        ...

    @property
    def lowest_note(self) -> PitchClassAndOctave:
        """
        Get/Set the lowest note of the current tuning system, where the first entry is
        the index within the pseudo octave and the second entry is the octave.
        """
        ...

    @lowest_note.setter
    def lowest_note(self, value: PitchClassAndOctave) -> None: ...

    def lowest_note_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "lowest_note".
        """
        ...

    @property
    def name(self) -> str:
        """Get/Set the name of the currently active tuning system."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    @property
    def note_tunings(self) -> list[float]:
        """Get/Set the currently active tuning system's note tunings, specified in Cents, where 100 Cents is one semi-tone in equal temperament."""
        ...

    @note_tunings.setter
    def note_tunings(self, value: list[float]) -> None: ...

    def note_tunings_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "note_tunings".
        """
        ...

    @property
    def number_of_notes_in_pseudo_octave(self) -> int:
        """Get the number of notes in the pseudo octave."""
        ...

    @property
    def pseudo_octave_in_cents(self) -> float:
        """Get the pseudo octave in cents for the currently active tuning system."""
        ...

    @property
    def reference_pitch(self) -> ReferencePitch:
        """Get/Set the reference pitch the currently active tuning system."""
        ...

    @reference_pitch.setter
    def reference_pitch(self, value: ReferencePitch) -> None: ...

    def reference_pitch_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "reference_pitch".
        """
        ...

    def remove_highest_note_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "highest_note".
        """
        ...

    def remove_lowest_note_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "lowest_note".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_note_tunings_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "note_tunings".
        """
        ...

    def remove_reference_pitch_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "reference_pitch".
        """
        ...

class PitchClassAndOctave:
    """This class represents a PitchClassAndOctave type."""

    def __init__(self, index_in_octave: int, octave: int) -> None: ...

    @property
    def index_in_octave(self) -> int:
        """A PitchClassAndOctave's index within the pseudo octave."""
        ...

    @property
    def octave(self) -> int:
        """A PitchClassAndOctave's octave."""
        ...

class ReferencePitch:
    """This class represents a ReferencePitch type."""

    def __init__(self, index_in_octave: int, octave: int, frequency: float) -> None: ...

    @property
    def frequency(self) -> float:
        """A ReferencePitch's frequency in Hz."""
        ...

    @property
    def index_in_octave(self) -> int:
        """A ReferencePitch's index within the pseudo octave."""
        ...

    @property
    def octave(self) -> int:
        """A ReferencePitch's octave."""
        ...

__all__ = ['TuningSystem', 'PitchClassAndOctave', 'ReferencePitch']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.GroovePool import GroovePool
    from Live.LomObject import LomObject



class Groove(LomObject):
    """This class represents a groove in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_quantization_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "quantization_amount" has changed.
        """
        ...

    def add_random_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "random_amount" has changed.
        """
        ...

    def add_timing_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "timing_amount" has changed.
        """
        ...

    def add_velocity_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "velocity_amount" has changed.
        """
        ...

    @property
    def base(self) -> Base:
        """Get/set the groove's base grid."""
        ...

    @base.setter
    def base(self, value: Base) -> None: ...

    @property
    def canonical_parent(self) -> GroovePool:
        """Get the canonical parent of the groove."""
        ...

    @property
    def name(self) -> str:
        """Read/write/listen access to the groove's name"""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    @property
    def quantization_amount(self) -> float:
        """Read/write/listen access to the groove's quantization amount."""
        ...

    @quantization_amount.setter
    def quantization_amount(self, value: float) -> None: ...

    def quantization_amount_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "quantization_amount".
        """
        ...

    @property
    def random_amount(self) -> float:
        """Read/write/listen access to the groove's random amount."""
        ...

    @random_amount.setter
    def random_amount(self, value: float) -> None: ...

    def random_amount_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "random_amount".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_quantization_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "quantization_amount".
        """
        ...

    def remove_random_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "random_amount".
        """
        ...

    def remove_timing_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "timing_amount".
        """
        ...

    def remove_velocity_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "velocity_amount".
        """
        ...

    @property
    def timing_amount(self) -> float:
        """Read/write/listen access to the groove's timing amount."""
        ...

    @timing_amount.setter
    def timing_amount(self, value: float) -> None: ...

    def timing_amount_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "timing_amount".
        """
        ...

    @property
    def velocity_amount(self) -> float:
        """Read/write/listen access to the groove's velocity amount."""
        ...

    @velocity_amount.setter
    def velocity_amount(self, value: float) -> None: ...

    def velocity_amount_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "velocity_amount".
        """
        ...

class Base(int):
    gb_four: int = 0
    gb_eight: int = 1
    gb_eight_triplet: int = 2
    gb_sixteen: int = 3
    gb_sixteen_triplet: int = 4
    gb_thirtytwo: int = 5
    count: int = 6

__all__ = ['Groove', 'Base']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Clip import Clip
    from Live.LomObject import LomObject



class Envelope(LomObject):
    """This class represents an automation or modulation envelope in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    @property
    def canonical_parent(self) -> Clip:
        """Get the canonical parent of the envelope."""
        ...

    def delete_events_in_range(self, start_time: float | None, end_time: float | None, /) -> None:
        """Deletes the events in the specified time range."""
        ...

    def events_in_range(self, start_time: float | None, end_time: float | None, /) -> EnvelopeEventVector:
        """Returns the events in the specified time range."""
        ...

    def insert_step(self, start_time: float | None, length: float | None, value: float | None, /) -> None:
        """Given a start time, a step length and a value, creates a step in the envelope."""
        ...

    def value_at_time(self, time: float | None, /) -> float:
        """Returns the parameter value at the specified time."""
        ...

class EnvelopeEvent:
    """This is a class that represents an envelope event."""

    def __init__(self, time: float, value: float, control_coefficients: EnvelopeEventControlCoefficients) -> None: ...

    @property
    def control_coefficients(self) -> EnvelopeEventControlCoefficients:
        ...

    @control_coefficients.setter
    def control_coefficients(self, value: EnvelopeEventControlCoefficients) -> None: ...

    @property
    def time(self) -> float:
        ...

    @time.setter
    def time(self, value: float) -> None: ...

    @property
    def value(self) -> float:
        ...

    @value.setter
    def value(self, value: float) -> None: ...

class EnvelopeEventControlCoefficients:
    """This class represents the control coefficients of an envelope event."""

    def __init__(self, x1: float, y1: float, x2: float, y2: float) -> None: ...

    @property
    def x1(self) -> float:
        ...

    @x1.setter
    def x1(self, value: float) -> None: ...

    @property
    def x2(self) -> float:
        ...

    @x2.setter
    def x2(self, value: float) -> None: ...

    @property
    def y1(self) -> float:
        ...

    @y1.setter
    def y1(self, value: float) -> None: ...

    @property
    def y2(self) -> float:
        ...

    @y2.setter
    def y2(self, value: float) -> None: ...

class EnvelopeEventVector(Vector[EnvelopeEvent]):
    """A container for holding envelope events."""

    def append(self, value: EnvelopeEvent | None, /) -> None:
        ...

    def extend(self, values: Iterable[EnvelopeEvent] | None, /) -> None:
        ...

__all__ = ['Envelope', 'EnvelopeEvent', 'EnvelopeEventControlCoefficients', 'EnvelopeEventVector']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.LomObject import LomObject
    from Live.RackDevice import RackDevice



class DrumPad(LomObject):
    """This class represents a drum group device pad in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "chains" has changed.
        """
        ...

    def add_mute_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mute" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "solo" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> RackDevice:
        """Get the canonical parent of the drum pad."""
        ...

    @property
    def chains(self) -> Vector:
        """Return const access to the list of chains in this drum pad."""
        ...

    def chains_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "chains".
        """
        ...

    def delete_all_chains(self) -> None:
        """Deletes all chains associated with a drum pad. This is equivalent to deleting a drum rack pad in Live."""
        ...

    @property
    def mute(self) -> bool:
        """Mute/unmute the pad."""
        ...

    @mute.setter
    def mute(self, value: bool) -> None: ...

    def mute_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mute".
        """
        ...

    @property
    def name(self) -> str:
        """Return const access to the drum pad's name. It depends on the contained chains."""
        ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    @property
    def note(self) -> int:
        """Get the MIDI note of the drum pad."""
        ...

    def remove_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "chains".
        """
        ...

    def remove_mute_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mute".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "solo".
        """
        ...

    @property
    def solo(self) -> bool:
        """Solo/unsolo the pad."""
        ...

    @solo.setter
    def solo(self, value: bool) -> None: ...

    def solo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "solo".
        """
        ...

__all__ = ['DrumPad']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.DeviceParameter import DeviceParameter



class CCFeedbackRule:
    """Structure to define feedback properties of MIDI mappings."""

    def __init__(self) -> None: ...

    @property
    def cc_no(self) -> int:
        ...

    @cc_no.setter
    def cc_no(self, value: int) -> None: ...

    @property
    def cc_value_map(self) -> tuple[int, ...]:
        ...

    @cc_value_map.setter
    def cc_value_map(self, value: tuple[int, ...]) -> None: ...

    @property
    def channel(self) -> int:
        ...

    @channel.setter
    def channel(self, value: int) -> None: ...

    @property
    def delay_in_ms(self) -> float:
        ...

    @delay_in_ms.setter
    def delay_in_ms(self, value: float) -> None: ...

    @property
    def enabled(self) -> bool:
        ...

    @enabled.setter
    def enabled(self, value: bool) -> None: ...

class MapMode(int):
    absolute: int = 0
    relative_signed_bit: int = 1
    relative_binary_offset: int = 2
    relative_two_compliment: int = 3
    relative_signed_bit2: int = 4
    absolute_14_bit: int = 5
    relative_smooth_signed_bit: int = 6
    relative_smooth_binary_offset: int = 7
    relative_smooth_two_compliment: int = 8
    relative_smooth_signed_bit2: int = 9

class NoteFeedbackRule:
    """Structure to define feedback properties of MIDI mappings."""

    def __init__(self) -> None: ...

    @property
    def channel(self) -> int:
        ...

    @channel.setter
    def channel(self, value: int) -> None: ...

    @property
    def delay_in_ms(self) -> float:
        ...

    @delay_in_ms.setter
    def delay_in_ms(self, value: float) -> None: ...

    @property
    def enabled(self) -> bool:
        ...

    @enabled.setter
    def enabled(self, value: bool) -> None: ...

    @property
    def note_no(self) -> int:
        ...

    @note_no.setter
    def note_no(self, value: int) -> None: ...

    @property
    def vel_map(self) -> tuple[int, ...]:
        ...

    @vel_map.setter
    def vel_map(self, value: tuple[int, ...]) -> None: ...

class PitchBendFeedbackRule:
    """Structure to define feedback properties of MIDI mappings."""

    def __init__(self) -> None: ...

    @property
    def channel(self) -> int:
        ...

    @channel.setter
    def channel(self, value: int) -> None: ...

    @property
    def delay_in_ms(self) -> float:
        ...

    @delay_in_ms.setter
    def delay_in_ms(self, value: float) -> None: ...

    @property
    def enabled(self) -> bool:
        ...

    @enabled.setter
    def enabled(self, value: bool) -> None: ...

    @property
    def value_pair_map(self) -> tuple[tuple, ...]:
        ...

    @value_pair_map.setter
    def value_pair_map(self, value: tuple[tuple, ...]) -> None: ...

def forward_midi_cc(script_handle: int | None, midi_map_handle: int | None, midi_channel: int | None, controller_number: int | None, should_consume_event: bool = True, /) -> bool:
    ...

def forward_midi_note(script_handle: int | None, midi_map_handle: int | None, midi_channel: int | None, note: int | None, should_consume_event: bool = True, /) -> bool:
    ...

def forward_midi_pitchbend(script_handle: int | None, midi_map_handle: int | None, midi_channel: int | None, /) -> bool:
    ...

def map_midi_cc(midi_map_handle: int | None, parameter: DeviceParameter | None, midi_channel: int | None, controller_number: int | None, map_mode: MapMode | None, avoid_takeover: bool | None, sensitivity: float = 1.0, /) -> bool:
    ...

def map_midi_cc_with_feedback_map(midi_map_handle: int | None, parameter: DeviceParameter | None, midi_channel: int | None, controller_number: int | None, map_mode: MapMode | None, feedback_rule: CCFeedbackRule | None, avoid_takeover: bool | None, sensitivity: float = 1.0, /) -> bool:
    ...

def map_midi_note(midi_map_handle: int | None, device_parameter: DeviceParameter | None, midi_channel: int | None, note: int | None, /) -> bool:
    ...

def map_midi_note_with_feedback_map(midi_map_handle: int | None, device_parameter: DeviceParameter | None, midi_channel: int | None, note: int | None, feedback_rule: NoteFeedbackRule | None, /) -> bool:
    ...

def map_midi_pitchbend(midi_map_handle: int | None, device_parameter: DeviceParameter | None, midi_channel: int | None, needs_takeover: bool | None, /) -> bool:
    ...

def map_midi_pitchbend_with_feedback_map(midi_map_handle: int | None, device_parameter: DeviceParameter | None, midi_channel: int | None, feedback_rule: PitchBendFeedbackRule | None, needs_takeover: bool | None, /) -> bool:
    ...

def send_feedback_for_parameter(midi_map_handle: int | None, device_parameter: DeviceParameter | None, /) -> None:
    ...

__all__ = ['CCFeedbackRule', 'MapMode', 'NoteFeedbackRule', 'PitchBendFeedbackRule', 'forward_midi_cc', 'forward_midi_note', 'forward_midi_pitchbend', 'map_midi_cc', 'map_midi_cc_with_feedback_map', 'map_midi_note', 'map_midi_note_with_feedback_map', 'map_midi_pitchbend', 'map_midi_pitchbend_with_feedback_map', 'send_feedback_for_parameter']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.LomObject import LomObject
    from Live.MaxDevice import MaxDevice
    from Live.Track import RoutingChannel, RoutingChannelVector, RoutingType, RoutingTypeVector



class DeviceIO(LomObject):
    """This class represents a specific input or output bus of a device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_available_routing_channels_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "available_routing_channels" has changed.
        """
        ...

    def add_available_routing_types_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "available_routing_types" has changed.
        """
        ...

    def add_routing_channel_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "routing_channel" has changed.
        """
        ...

    def add_routing_type_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "routing_type" has changed.
        """
        ...

    @property
    def available_routing_channels(self) -> RoutingChannelVector:
        """Return a list of channels for this IO endpoint."""
        ...

    def available_routing_channels_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "available_routing_channels".
        """
        ...

    @property
    def available_routing_types(self) -> RoutingTypeVector:
        """Return a list of available routing types for this IO endpoint."""
        ...

    def available_routing_types_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "available_routing_types".
        """
        ...

    @property
    def canonical_parent(self) -> MaxDevice:
        """Get the canonical parent of the device IO."""
        ...

    @property
    def default_external_routing_channel_is_none(self) -> bool:
        """Get and set whether the default routing channel for External routing types is none."""
        ...

    @default_external_routing_channel_is_none.setter
    def default_external_routing_channel_is_none(self, value: bool) -> None: ...

    def remove_available_routing_channels_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "available_routing_channels".
        """
        ...

    def remove_available_routing_types_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "available_routing_types".
        """
        ...

    def remove_routing_channel_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "routing_channel".
        """
        ...

    def remove_routing_type_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "routing_type".
        """
        ...

    @property
    def routing_channel(self) -> RoutingChannel:
        """
        Get and set the current routing channel.
        Raises ValueError if the channel isn't one of the current values in
        available_routing_channels.
        """
        ...

    @routing_channel.setter
    def routing_channel(self, value: RoutingChannel) -> None: ...

    def routing_channel_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "routing_channel".
        """
        ...

    @property
    def routing_type(self) -> RoutingType:
        """
        Get and set the current routing type.
        Raises ValueError if the type isn't one of the current values in
        available_routing_types.
        """
        ...

    @routing_type.setter
    def routing_type(self, value: RoutingType) -> None: ...

    def routing_type_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "routing_type".
        """
        ...

__all__ = ['DeviceIO']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Clip import Clip
    from Live.LomObject import LomObject
    from Live.Track import Track



class ClipSlot(LomObject):
    """This class represents an entry in Lives Session view matrix."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color_index" has changed.
        """
        ...

    def add_color_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color" has changed.
        """
        ...

    def add_controls_other_clips_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "controls_other_clips" has changed.
        """
        ...

    def add_has_clip_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_clip" has changed.
        """
        ...

    def add_has_stop_button_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_stop_button" has changed.
        """
        ...

    def add_is_triggered_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_triggered" has changed.
        """
        ...

    def add_playing_status_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "playing_status" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the ClipSlot."""
        ...

    @property
    def clip(self) -> Clip:
        """Returns the Clip which this clipslots currently owns. Might be None."""
        ...

    @property
    def color(self) -> int:
        """Returns the canonical color for the clip slot or None if it does not exist."""
        ...

    def color_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color".
        """
        ...

    @property
    def color_index(self) -> int:
        """Returns the canonical color index for the clip slot or None if it does not exist."""
        ...

    def color_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color_index".
        """
        ...

    @property
    def controls_other_clips(self) -> bool:
        """
        Returns true if firing this slot will fire clips in other slots.
        Can only be true for slots in group tracks.
        """
        ...

    def controls_other_clips_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "controls_other_clips".
        """
        ...

    def create_audio_clip(self, file_path: str | None, /) -> Clip:
        """
        Creates an audio clip referencing the file at the given absolute path in the slot.
        Throws an error when called on non-empty slots or slots in non-audio or frozen tracks, or when the path doesn't point at a valid audio file.
        """
        ...

    def create_clip(self, pages: float | None, /) -> Clip:
        """
        Creates an empty clip with the given length in the slot.
        Throws an error when called on non-empty slots or slots in non-MIDI tracks.
        """
        ...

    def delete_clip(self) -> None:
        """
        Removes the clip contained in the slot.
        Raises an exception if the slot was empty.
        """
        ...

    def duplicate_clip_to(self, target_clip_slot: ClipSlot | None, /) -> None:
        """
        Duplicates the slot's clip to the passed in target slot.
        Overrides the target's clip if it's not empty.
        Raises an exception if the (source) slot itself is empty, or if source and
        target have different track types (audio vs. MIDI). Also raises if the source
        or target slot is in a group track (so called group slot).
        """
        ...

    def fire(self) -> None:
        """
        Fire a Clip if this Clipslot owns one, else trigger the stop button,
        if we have one.
        """
        ...

    @property
    def has_clip(self) -> bool:
        """Returns true if this Clipslot owns a Clip."""
        ...

    def has_clip_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_clip".
        """
        ...

    @property
    def has_stop_button(self) -> bool:
        """
        Get/Set if this Clip has a stop button, which will, if fired, stop any
        other Clip that is currently playing the Track we do belong to.
        """
        ...

    @has_stop_button.setter
    def has_stop_button(self, value: bool) -> None: ...

    def has_stop_button_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_stop_button".
        """
        ...

    @property
    def is_group_slot(self) -> bool:
        """Returns whether this clip slot is a group track slot (group slot)."""
        ...

    @property
    def is_playing(self) -> bool:
        """Returns whether the clip associated with the slot is playing."""
        ...

    @property
    def is_recording(self) -> bool:
        """Returns whether the clip associated with the slot is recording."""
        ...

    @property
    def is_triggered(self) -> bool:
        """Const access to the triggering state of the clip slot."""
        ...

    def is_triggered_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_triggered".
        """
        ...

    @property
    def playing_status(self) -> ClipSlotPlayingState:
        """
        Const access to the playing state of the clip slot.
        Can be either stopped, playing, or recording.
        """
        ...

    def playing_status_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "playing_status".
        """
        ...

    def remove_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color_index".
        """
        ...

    def remove_color_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color".
        """
        ...

    def remove_controls_other_clips_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "controls_other_clips".
        """
        ...

    def remove_has_clip_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_clip".
        """
        ...

    def remove_has_stop_button_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_stop_button".
        """
        ...

    def remove_is_triggered_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_triggered".
        """
        ...

    def remove_playing_status_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "playing_status".
        """
        ...

    def set_fire_button_state(self, state: bool | None, /) -> None:
        """Set the clipslot's fire button state directly. Supports all launch modes."""
        ...

    def stop(self) -> None:
        """
        Stop playing the contained Clip, if there is a Clip and its currently
        playing.
        """
        ...

    @property
    def will_record_on_start(self) -> bool:
        """returns true if the clip slot will record on being fired."""
        ...

class ClipSlotPlayingState(int):
    stopped: int = 0
    started: int = 1
    recording: int = 2

__all__ = ['ClipSlot', 'ClipSlotPlayingState']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import StringVector
    from Live.Device import Device



class CcControlDevice(Device):
    """This class represents a CcControl device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_custom_bool_target_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_bool_target" has changed.
        """
        ...

    def add_custom_float_target_0_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_0" has changed.
        """
        ...

    def add_custom_float_target_10_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_10" has changed.
        """
        ...

    def add_custom_float_target_11_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_11" has changed.
        """
        ...

    def add_custom_float_target_1_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_1" has changed.
        """
        ...

    def add_custom_float_target_2_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_2" has changed.
        """
        ...

    def add_custom_float_target_3_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_3" has changed.
        """
        ...

    def add_custom_float_target_4_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_4" has changed.
        """
        ...

    def add_custom_float_target_5_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_5" has changed.
        """
        ...

    def add_custom_float_target_6_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_6" has changed.
        """
        ...

    def add_custom_float_target_7_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_7" has changed.
        """
        ...

    def add_custom_float_target_8_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_8" has changed.
        """
        ...

    def add_custom_float_target_9_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "custom_float_target_9" has changed.
        """
        ...

    @property
    def custom_bool_target(self) -> int:
        """Return the custom bool target"""
        ...

    @custom_bool_target.setter
    def custom_bool_target(self, value: int) -> None: ...

    def custom_bool_target_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_bool_target".
        """
        ...

    @property
    def custom_bool_target_list(self) -> StringVector:
        """Return the custom bool target list"""
        ...

    @property
    def custom_float_target_0(self) -> int:
        """Return the custom float target 0"""
        ...

    @custom_float_target_0.setter
    def custom_float_target_0(self, value: int) -> None: ...

    def custom_float_target_0_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_0".
        """
        ...

    @property
    def custom_float_target_0_list(self) -> StringVector:
        """Return the custom float target 0 list"""
        ...

    @property
    def custom_float_target_1(self) -> int:
        """Return the custom float target 1"""
        ...

    @custom_float_target_1.setter
    def custom_float_target_1(self, value: int) -> None: ...

    @property
    def custom_float_target_10(self) -> int:
        """Return the custom float target 10"""
        ...

    @custom_float_target_10.setter
    def custom_float_target_10(self, value: int) -> None: ...

    def custom_float_target_10_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_10".
        """
        ...

    @property
    def custom_float_target_10_list(self) -> StringVector:
        """Return the custom float target 10 list"""
        ...

    @property
    def custom_float_target_11(self) -> int:
        """Return the custom float target 11"""
        ...

    @custom_float_target_11.setter
    def custom_float_target_11(self, value: int) -> None: ...

    def custom_float_target_11_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_11".
        """
        ...

    @property
    def custom_float_target_11_list(self) -> StringVector:
        """Return the custom float target 11 list"""
        ...

    def custom_float_target_1_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_1".
        """
        ...

    @property
    def custom_float_target_1_list(self) -> StringVector:
        """Return the custom float target 1 list"""
        ...

    @property
    def custom_float_target_2(self) -> int:
        """Return the custom float target 2"""
        ...

    @custom_float_target_2.setter
    def custom_float_target_2(self, value: int) -> None: ...

    def custom_float_target_2_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_2".
        """
        ...

    @property
    def custom_float_target_2_list(self) -> StringVector:
        """Return the custom float target 2 list"""
        ...

    @property
    def custom_float_target_3(self) -> int:
        """Return the custom float target 3"""
        ...

    @custom_float_target_3.setter
    def custom_float_target_3(self, value: int) -> None: ...

    def custom_float_target_3_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_3".
        """
        ...

    @property
    def custom_float_target_3_list(self) -> StringVector:
        """Return the custom float target 3 list"""
        ...

    @property
    def custom_float_target_4(self) -> int:
        """Return the custom float target 4"""
        ...

    @custom_float_target_4.setter
    def custom_float_target_4(self, value: int) -> None: ...

    def custom_float_target_4_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_4".
        """
        ...

    @property
    def custom_float_target_4_list(self) -> StringVector:
        """Return the custom float target 4 list"""
        ...

    @property
    def custom_float_target_5(self) -> int:
        """Return the custom float target 5"""
        ...

    @custom_float_target_5.setter
    def custom_float_target_5(self, value: int) -> None: ...

    def custom_float_target_5_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_5".
        """
        ...

    @property
    def custom_float_target_5_list(self) -> StringVector:
        """Return the custom float target 5 list"""
        ...

    @property
    def custom_float_target_6(self) -> int:
        """Return the custom float target 6"""
        ...

    @custom_float_target_6.setter
    def custom_float_target_6(self, value: int) -> None: ...

    def custom_float_target_6_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_6".
        """
        ...

    @property
    def custom_float_target_6_list(self) -> StringVector:
        """Return the custom float target 6 list"""
        ...

    @property
    def custom_float_target_7(self) -> int:
        """Return the custom float target 7"""
        ...

    @custom_float_target_7.setter
    def custom_float_target_7(self, value: int) -> None: ...

    def custom_float_target_7_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_7".
        """
        ...

    @property
    def custom_float_target_7_list(self) -> StringVector:
        """Return the custom float target 7 list"""
        ...

    @property
    def custom_float_target_8(self) -> int:
        """Return the custom float target 8"""
        ...

    @custom_float_target_8.setter
    def custom_float_target_8(self, value: int) -> None: ...

    def custom_float_target_8_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_8".
        """
        ...

    @property
    def custom_float_target_8_list(self) -> StringVector:
        """Return the custom float target 8 list"""
        ...

    @property
    def custom_float_target_9(self) -> int:
        """Return the custom float target 9"""
        ...

    @custom_float_target_9.setter
    def custom_float_target_9(self, value: int) -> None: ...

    def custom_float_target_9_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "custom_float_target_9".
        """
        ...

    @property
    def custom_float_target_9_list(self) -> StringVector:
        """Return the custom float target 9 list"""
        ...

    def remove_custom_bool_target_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_bool_target".
        """
        ...

    def remove_custom_float_target_0_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_0".
        """
        ...

    def remove_custom_float_target_10_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_10".
        """
        ...

    def remove_custom_float_target_11_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_11".
        """
        ...

    def remove_custom_float_target_1_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_1".
        """
        ...

    def remove_custom_float_target_2_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_2".
        """
        ...

    def remove_custom_float_target_3_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_3".
        """
        ...

    def remove_custom_float_target_4_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_4".
        """
        ...

    def remove_custom_float_target_5_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_5".
        """
        ...

    def remove_custom_float_target_6_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_6".
        """
        ...

    def remove_custom_float_target_7_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_7".
        """
        ...

    def remove_custom_float_target_8_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_8".
        """
        ...

    def remove_custom_float_target_9_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "custom_float_target_9".
        """
        ...

    def resend(self) -> None:
        """Resend all CC values."""
        ...

__all__ = ['CcControlDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import StringVector
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.Track import Track



class SpectralResonatorDevice(Device):
    """This class represents a Spectral Resonator device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_frequency_dial_mode_list_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "frequency_dial_mode_list" has changed.
        """
        ...

    def add_frequency_dial_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "frequency_dial_mode" has changed.
        """
        ...

    def add_midi_gate_list_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "midi_gate_list" has changed.
        """
        ...

    def add_midi_gate_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "midi_gate" has changed.
        """
        ...

    def add_mod_mode_list_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_mode_list" has changed.
        """
        ...

    def add_mod_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_mode" has changed.
        """
        ...

    def add_mono_poly_list_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mono_poly_list" has changed.
        """
        ...

    def add_mono_poly_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mono_poly" has changed.
        """
        ...

    def add_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pitch_bend_range" has changed.
        """
        ...

    def add_pitch_mode_list_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pitch_mode_list" has changed.
        """
        ...

    def add_pitch_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pitch_mode" has changed.
        """
        ...

    def add_polyphony_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "polyphony" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    @property
    def frequency_dial_mode(self) -> int:
        """Return the current frequency dial mode index"""
        ...

    @frequency_dial_mode.setter
    def frequency_dial_mode(self, value: int) -> None: ...

    def frequency_dial_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "frequency_dial_mode".
        """
        ...

    @property
    def frequency_dial_mode_list(self) -> StringVector:
        """Return the current frequency dial mode list"""
        ...

    def frequency_dial_mode_list_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "frequency_dial_mode_list".
        """
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def midi_gate(self) -> int:
        """Return the current midi gate index"""
        ...

    @midi_gate.setter
    def midi_gate(self, value: int) -> None: ...

    def midi_gate_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "midi_gate".
        """
        ...

    @property
    def midi_gate_list(self) -> StringVector:
        """Return the current midi gate list"""
        ...

    def midi_gate_list_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "midi_gate_list".
        """
        ...

    @property
    def mod_mode(self) -> int:
        """Return the current mod mode index"""
        ...

    @mod_mode.setter
    def mod_mode(self, value: int) -> None: ...

    def mod_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_mode".
        """
        ...

    @property
    def mod_mode_list(self) -> StringVector:
        """Return the current mod mode list"""
        ...

    def mod_mode_list_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_mode_list".
        """
        ...

    @property
    def mono_poly(self) -> int:
        """Return the current mono poly mode index"""
        ...

    @mono_poly.setter
    def mono_poly(self, value: int) -> None: ...

    def mono_poly_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mono_poly".
        """
        ...

    @property
    def mono_poly_list(self) -> StringVector:
        """Return the current mono poly mode list"""
        ...

    def mono_poly_list_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mono_poly_list".
        """
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    @property
    def pitch_bend_range(self) -> int:
        """Return the current pitch bend range"""
        ...

    @pitch_bend_range.setter
    def pitch_bend_range(self, value: int) -> None: ...

    def pitch_bend_range_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pitch_bend_range".
        """
        ...

    @property
    def pitch_mode(self) -> int:
        """Return the current pitch mode index"""
        ...

    @pitch_mode.setter
    def pitch_mode(self, value: int) -> None: ...

    def pitch_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pitch_mode".
        """
        ...

    @property
    def pitch_mode_list(self) -> StringVector:
        """Return the current pitch mode list"""
        ...

    def pitch_mode_list_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pitch_mode_list".
        """
        ...

    @property
    def polyphony(self) -> int:
        """Return the current polyphony"""
        ...

    @polyphony.setter
    def polyphony(self, value: int) -> None: ...

    def polyphony_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "polyphony".
        """
        ...

    def remove_frequency_dial_mode_list_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "frequency_dial_mode_list".
        """
        ...

    def remove_frequency_dial_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "frequency_dial_mode".
        """
        ...

    def remove_midi_gate_list_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "midi_gate_list".
        """
        ...

    def remove_midi_gate_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "midi_gate".
        """
        ...

    def remove_mod_mode_list_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_mode_list".
        """
        ...

    def remove_mod_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_mode".
        """
        ...

    def remove_mono_poly_list_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mono_poly_list".
        """
        ...

    def remove_mono_poly_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mono_poly".
        """
        ...

    def remove_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pitch_bend_range".
        """
        ...

    def remove_pitch_mode_list_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pitch_mode_list".
        """
        ...

    def remove_pitch_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pitch_mode".
        """
        ...

    def remove_polyphony_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "polyphony".
        """
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def view(self) -> Device.View:
        """Representing the view aspects of a device."""
        ...

__all__ = ['SpectralResonatorDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Device import Device
    from Live.LomObject import LomObject



class Browser(LomObject):
    """This class represents the live browser data base."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_filter_type_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "filter_type" has changed.
        """
        ...

    def add_full_refresh_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "full_refresh" has changed.
        """
        ...

    def add_hotswap_target_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "hotswap_target" has changed.
        """
        ...

    @property
    def audio_effects(self) -> BrowserItem:
        """Returns a browser item with access to all the Audio Effects content."""
        ...

    @property
    def clips(self) -> BrowserItem:
        """Returns a browser item with access to all the Clips content."""
        ...

    @property
    def colors(self) -> BrowserItemVector:
        """Returns a list of browser items containing the configured colors."""
        ...

    @property
    def current_project(self) -> BrowserItem:
        """Returns a browser item with access to all the Current Project content."""
        ...

    @property
    def drums(self) -> BrowserItem:
        """Returns a browser item with access to all the Drums content."""
        ...

    @property
    def filter_type(self) -> int:
        """Bang triggered when the hotswap target has changed."""
        ...

    @filter_type.setter
    def filter_type(self, value: int) -> None: ...

    def filter_type_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "filter_type".
        """
        ...

    def full_refresh_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "full_refresh".
        """
        ...

    @property
    def hotswap_target(self) -> Device | None:
        """Bang triggered when the hotswap target has changed."""
        ...

    @hotswap_target.setter
    def hotswap_target(self, value: Device | None) -> None: ...

    def hotswap_target_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "hotswap_target".
        """
        ...

    @property
    def instruments(self) -> BrowserItem:
        """Returns a browser item with access to all the Instruments content."""
        ...

    @property
    def legacy_libraries(self) -> BrowserItemVector:
        """Returns a list of browser items containing the installed legacy libraries. The list is always empty as legacy library handling has been removed."""
        ...

    def load_item(self, item: BrowserItem | None, /) -> None:
        """Loads the provided browser item."""
        ...

    @property
    def max_for_live(self) -> BrowserItem:
        """Returns a browser item with access to all the Max For Live content."""
        ...

    @property
    def midi_effects(self) -> BrowserItem:
        """Returns a browser item with access to all the Midi Effects content."""
        ...

    @property
    def packs(self) -> BrowserItem:
        """Returns a browser item with access to all the Packs content."""
        ...

    @property
    def plugins(self) -> BrowserItem:
        """Returns a browser item with access to all the Plugins content."""
        ...

    def preview_item(self, item: BrowserItem | None, /) -> None:
        """Previews the provided browser item."""
        ...

    def relation_to_hotswap_target(self, item: BrowserItem | None, /) -> Relation:
        """Returns the relation between the given browser item and the current hotswap target"""
        ...

    def remove_filter_type_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "filter_type".
        """
        ...

    def remove_full_refresh_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "full_refresh".
        """
        ...

    def remove_hotswap_target_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "hotswap_target".
        """
        ...

    @property
    def samples(self) -> BrowserItem:
        """Returns a browser item with access to all the Samples content."""
        ...

    @property
    def sounds(self) -> BrowserItem:
        """Returns a browser item with access to all the Sounds content."""
        ...

    def stop_preview(self) -> None:
        """Stop the current preview."""
        ...

    @property
    def user_folders(self) -> BrowserItemVector:
        """Returns a list of browser items containing all the user folders."""
        ...

    @property
    def user_library(self) -> BrowserItem:
        """Returns a browser item with access to all the User Library content."""
        ...

class BrowserItem:
    """This class represents an item of the browser hierarchy."""

    @property
    def children(self) -> BrowserItemVector:
        """Const access to the descendants of this browser item."""
        ...

    @property
    def is_device(self) -> bool:
        """Indicates if the browser item represents a device."""
        ...

    @property
    def is_folder(self) -> bool:
        """Indicates if the browser item represents folder."""
        ...

    @property
    def is_loadable(self) -> bool:
        """True if item can be loaded via the Browser's 'load_item' method."""
        ...

    @property
    def is_selected(self) -> bool:
        """True if the item is ancestor of or the actual selection."""
        ...

    @property
    def iter_children(self) -> BrowserItemIterator:
        """Const iterable access to the descendants of this browser item."""
        ...

    @property
    def name(self) -> str:
        """Const access to the canonical display name of this browser item."""
        ...

    @property
    def source(self) -> str:
        """Specifies where does item come from -- i.e. Live pack, user library..."""
        ...

    @property
    def uri(self) -> str:
        """The uri describes a unique identifier for a browser item."""
        ...

class BrowserItemIterator(Iterable[BrowserItem]):
    """This class iterates over children of another BrowserItem."""

class BrowserItemVector(Vector[BrowserItem]):
    """A container for returning browser items from Live."""

    def append(self, value: BrowserItem | None, /) -> None:
        ...

    def extend(self, values: Iterable[BrowserItem] | None, /) -> None:
        ...

class FilterType(int):
    disabled: int = -1
    hotswap_off: int = 0
    instrument_hotswap: int = 1
    audio_effect_hotswap: int = 2
    midi_effect_hotswap: int = 3
    drum_pad_hotswap: int = 4
    midi_track_devices: int = 5
    samples: int = 6
    count: int = 7

class Relation(int):
    ancestor: int = 0
    equal: int = 1
    descendant: int = 2
    none: int = 3

__all__ = ['Browser', 'BrowserItem', 'BrowserItemIterator', 'BrowserItemVector', 'FilterType', 'Relation']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.Base import StringVector, Text, Vector
    from Live.Browser import Browser
    from Live.LomObject import LomObject
    from Live.Song import Song



class Application(LomObject):
    """This class represents the Live application."""

    class View(LomObject):
        """This class represents the view aspects of the Live application."""

        class NavDirection(int):
            up: int = 0
            down: int = 1
            left: int = 2
            right: int = 3

        @property
        def _live_ptr(self) -> int:
            ...

        def add_browse_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "browse_mode" has changed.
            """
            ...

        def add_focused_document_view_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "focused_document_view" has changed.
            """
            ...

        def add_is_view_visible_listener(self, view_name: str | None, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "is_view_visible" has changed.
            """
            ...

        def add_view_focus_changed_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "view_focus_changed" has changed.
            """
            ...

        def available_main_views(self) -> StringVector:
            """
            Return a list of strings with the available subcomponent views, which
            is to be specified, when using the rest of this classes functions.
            A 'subcomponent view' is a main view component of a document view, like
            the Session view, the Arranger or Detailview and so on...
            """
            ...

        @property
        def browse_mode(self) -> bool:
            """Return true if HotSwap mode is active for any target."""
            ...

        def browse_mode_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "browse_mode".
            """
            ...

        @property
        def canonical_parent(self) -> Application:
            """Get the canonical parent of the application view."""
            ...

        def focus_view(self, view: str | None, /) -> None:
            """Show and focus one through the identifier string specified view."""
            ...

        @property
        def focused_document_view(self) -> str:
            """
            Return the name of the document view ('Session' or 'Arranger')
            shown in the currently selected window.
            """
            ...

        def focused_document_view_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "focused_document_view".
            """
            ...

        def hide_view(self, view_name: str | None, /) -> None:
            """Hide one through the identifier string specified view."""
            ...

        def is_view_visible(self, identifier: str | None, main_window_only: bool = True, /) -> bool:
            """
            Return true if the through the identifier string specified view is currently
            visible. If main_window_only is set to False, this will also check in second
            window. Notifications from the second window are not yet supported.
            """
            ...

        def is_view_visible_has_listener(self, view_name: str | None, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "is_view_visible".
            """
            ...

        def remove_browse_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "browse_mode".
            """
            ...

        def remove_focused_document_view_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "focused_document_view".
            """
            ...

        def remove_is_view_visible_listener(self, view_name: str | None, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "is_view_visible".
            """
            ...

        def remove_view_focus_changed_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "view_focus_changed".
            """
            ...

        def scroll_view(self, direction: int | None, view_name: str | None, modifier_pressed: bool | None, /) -> None:
            """
            Scroll through the identifier string specified view into the given
            direction, if possible. Will silently return if the specified view
            can not perform the requested action.
            """
            ...

        def show_view(self, view: str | None, /) -> None:
            """
            Show one through the identifier string specified view. Will throw a
            runtime error if this is called in Live's initialization scope.
            """
            ...

        def toggle_browse(self) -> None:
            """
            Reveals the device chain, the browser and starts hot swap for
            the selected device. Calling this function again stops hot swap.
            """
            ...

        def view_focus_changed_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "view_focus_changed".
            """
            ...

        def zoom_view(self, direction: int | None, view_name: str | None, modifier_pressed: bool | None, /) -> None:
            """
            Zoom through the identifier string specified view into the given
            direction, if possible. Will silently return if the specified view
            can not perform the requested action.
            """
            ...

    def add_average_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "average_process_usage" has changed.
        """
        ...

    def add_control_surfaces_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "control_surfaces" has changed.
        """
        ...

    def add_open_dialog_count_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "open_dialog_count" has changed.
        """
        ...

    def add_peak_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "peak_process_usage" has changed.
        """
        ...

    def add_unavailable_features_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "unavailable_features" has changed.
        """
        ...

    @property
    def average_process_usage(self) -> float:
        """Reports Live's average CPU load."""
        ...

    def average_process_usage_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "average_process_usage".
        """
        ...

    @property
    def browser(self) -> Browser:
        """Returns an interface to the browser."""
        ...

    @property
    def canonical_parent(self) -> None:
        """Returns the canonical parent of the application."""
        ...

    @property
    def control_surfaces(self) -> Vector[object]:
        """
        Const access to a list of the control surfaces selected in preferences, in the same order.
        The list contains None if no control surface is active at that index.
        """
        ...

    def control_surfaces_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "control_surfaces".
        """
        ...

    @property
    def current_dialog_button_count(self) -> int:
        """Number of buttons on the current dialog."""
        ...

    @property
    def current_dialog_message(self) -> str:
        """Text of the last dialog that appeared; Empty if all dialogs just disappeared."""
        ...

    def get_bugfix_version(self) -> int:
        """Returns an integer representing the bugfix version of Live."""
        ...

    def get_build_id(self) -> str:
        """Returns a string identifying the build."""
        ...

    def get_document(self) -> Song:
        """Returns the current Live Set."""
        ...

    def get_major_version(self) -> int:
        """Returns an integer representing the major version of Live."""
        ...

    def get_minor_version(self) -> int:
        """Returns an integer representing the minor version of Live."""
        ...

    def get_variant(self) -> str:
        """Returns one of the strings in Live.Application.Variants."""
        ...

    def get_version_string(self) -> str:
        """Returns the full version string of Live."""
        ...

    def has_option(self, option_name: str | None, /) -> bool:
        """Returns True if the given entry exists in Options.txt, False otherwise."""
        ...

    @property
    def number_of_push_apps_running(self) -> int:
        """Returns the number of connected Push apps."""
        ...

    @property
    def open_dialog_count(self) -> int:
        """The number of open dialogs in Live. 0 if not dialog is open."""
        ...

    def open_dialog_count_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "open_dialog_count".
        """
        ...

    @property
    def peak_process_usage(self) -> float:
        """Reports Live's peak CPU load."""
        ...

    def peak_process_usage_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "peak_process_usage".
        """
        ...

    def press_current_dialog_button(self, index: int | None, /) -> None:
        """Press a button, by index, on the current message box."""
        ...

    def remove_average_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "average_process_usage".
        """
        ...

    def remove_control_surfaces_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "control_surfaces".
        """
        ...

    def remove_open_dialog_count_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "open_dialog_count".
        """
        ...

    def remove_peak_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "peak_process_usage".
        """
        ...

    def remove_unavailable_features_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "unavailable_features".
        """
        ...

    def show_message(self, text: Text | None, buttons: MessageButtons | int = 0, enable_markup: bool = False, show_success_icon: bool = False, /) -> int:
        """Shows a message box, returning the position of the pressed button."""
        ...

    def show_on_the_fly_message(self, message: str | None, buttons: MessageButtons | int = 0, enable_markup: bool = False, show_success_icon: bool = False, push_dialog_type: PushDialogType | int = 0, /) -> int:
        """Same as show_message, but for when there is no predefined Text object."""
        ...

    @property
    def unavailable_features(self) -> UnavailableFeatureVector:
        """List of features that are unavailable due to limitations of the current Live edition."""
        ...

    def unavailable_features_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "unavailable_features".
        """
        ...

    @property
    def view(self) -> View:
        """Returns the applications view component."""
        ...

class ControlDescription:
    """Describes a control present in a control surface proxy"""

    def __init__(self) -> None: ...

    @property
    def id(self) -> int:
        ...

    @property
    def name(self) -> str:
        ...

class ControlDescriptionVector(Vector[ControlDescription]):
    """A container for returning control descriptions."""

    def append(self, value: ControlDescription | None, /) -> None:
        ...

    def extend(self, values: Iterable[ControlDescription] | None, /) -> None:
        ...

class ControlSurfaceProxy:
    """Represents a control surface running in a different process. For use by M4L"""

    def add_control_values_arrived_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "control_values_arrived" has changed.
        """
        ...

    def add_midi_received_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "midi_received" has changed.
        """
        ...

    def add_pad_layout_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pad_layout" has changed.
        """
        ...

    @property
    def control_descriptions(self) -> ControlDescriptionVector:
        ...

    def control_values_arrived_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "control_values_arrived".
        """
        ...

    def enable_receive_midi(self, enabled: bool | None, /) -> None:
        ...

    def fetch_received_midi_messages(self) -> tuple[tuple[int, ...], ...]:
        ...

    def fetch_received_values(self) -> tuple[tuple[int, Any], ...]:
        ...

    def grab_control(self, control: int | None, /) -> None:
        ...

    def midi_received_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "midi_received".
        """
        ...

    @property
    def pad_layout(self) -> str:
        """The layout of pads on Push."""
        ...

    def pad_layout_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pad_layout".
        """
        ...

    def release_control(self, control: int | None, /) -> None:
        ...

    def remove_control_values_arrived_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "control_values_arrived".
        """
        ...

    def remove_midi_received_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "midi_received".
        """
        ...

    def remove_pad_layout_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pad_layout".
        """
        ...

    def send_midi(self, midi_event_bytes: tuple[int, ...] | None, /) -> None:
        ...

    def send_value(self, value: tuple[Any, ...] | None, /) -> None:
        ...

    def subscribe_to_control(self, control: int | None, /) -> None:
        ...

    @property
    def type_name(self) -> str:
        ...

    def unsubscribe_from_control(self, control: int | None, /) -> None:
        ...

class MessageButtons(int):
    """Specifies the characteristics of the message box, e.g. which buttons to show."""
    OK_BUTTON: int = 0
    OK_NEW_SET_BUTTON: int = 1
    OK_RETRY_BUTTON: int = 2
    SAVE_DONT_SAVE_BUTTON: int = 3
    OK_ACCOUNT_BUTTON: int = 4
    OK_PURCHASE_BUTTON: int = 5

class PushDialogType(int):
    """Specifies the dialog type for Push."""
    MESSAGE_BOX: int = 0
    OUT_OF_UNLOCKS_DIALOG: int = 5
    RENT_TO_OWN_LICENSE_EXPIRED_DIALOG: int = 7

class UnavailableFeature(int):
    note_velocity_ranges_and_probabilities: int = 0

class UnavailableFeatureVector(Vector[UnavailableFeature]):
    """A container for returning unavailable features."""

    def append(self, value: UnavailableFeature | None, /) -> None:
        ...

    def extend(self, values: Iterable[UnavailableFeature] | None, /) -> None:
        ...

class Variants:
    """Holds strings representing what type of Live is running."""
    BETA: str = "Beta"
    INTRO: str = "Intro"
    LITE: str = "Lite"
    STANDARD: str = "Standard"
    SUITE: str = "Suite"
    TRIAL: str = "Trial"

def combine_apcs() -> bool:
    """Returns true if multiple APCs should be combined."""
    ...

def encrypt_challenge(dongle1: int | None, dongle2: int | None, key_index: int = 0, /) -> tuple[int, ...]:
    """Returns an encrypted challenge based on the TEA algortithm"""
    ...

def encrypt_challenge2(challenge: int | None, /) -> int:
    """Returns the UMAC hash for the given challenge."""
    ...

def get_application() -> Application:
    """Returns the application instance."""
    ...

def get_random_int(min_value: int | None, max_value: int | None, /) -> int:
    """Returns a random integer from the given range."""
    ...

__all__ = ['Application', 'ControlDescription', 'ControlDescriptionVector', 'ControlSurfaceProxy', 'MessageButtons', 'PushDialogType', 'UnavailableFeature', 'UnavailableFeatureVector', 'Variants', 'combine_apcs', 'encrypt_challenge', 'encrypt_challenge2', 'get_application', 'get_random_int']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.Base import StringVector, Vector
    from Live.Chain import Chain
    from Live.Clip import Clip
    from Live.ClipSlot import ClipSlot
    from Live.Device import Device
    from Live.LomObject import LomObject
    from Live.MixerDevice import MixerDevice
    from Live.Song import Song
    from Live.TakeLane import TakeLane



class Track(DeviceContainer):
    """
    This class represents a track in Live. It can be either an Audio
    track, a MIDI Track, a Return Track or the Main track. The Main
    Track and at least one Audio or MIDI track will be always present.
    Return Tracks are optional.
    """

    class View(LomObject):
        """Representing the view aspects of a Track."""

        @property
        def _live_ptr(self) -> int:
            ...

        def add_device_insert_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "device_insert_mode" has changed.
            """
            ...

        def add_is_collapsed_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "is_collapsed" has changed.
            """
            ...

        def add_selected_device_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_device" has changed.
            """
            ...

        @property
        def canonical_parent(self) -> Track:
            """Get the canonical parent of the track view."""
            ...

        @property
        def device_insert_mode(self) -> bool:
            """Get/Listen the device insertion mode of the track.  By default, it will insert devices at the end, but it can be changed to make it relative to current selection."""
            ...

        @device_insert_mode.setter
        def device_insert_mode(self, value: bool) -> None: ...

        def device_insert_mode_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "device_insert_mode".
            """
            ...

        @property
        def is_collapsed(self) -> bool:
            """Get/Set/Listen if the track is shown collapsed in the arranger view."""
            ...

        @is_collapsed.setter
        def is_collapsed(self, value: bool) -> None: ...

        def is_collapsed_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "is_collapsed".
            """
            ...

        def remove_device_insert_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "device_insert_mode".
            """
            ...

        def remove_is_collapsed_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "is_collapsed".
            """
            ...

        def remove_selected_device_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_device".
            """
            ...

        def select_instrument(self) -> bool:
            """Selects the track's instrument if it has one."""
            ...

        @property
        def selected_device(self) -> Device:
            """Get/Set/Listen the insertion mode of the device.  While in insertion mode, loading new devices from the browser will place devices at the selected position."""
            ...

        def selected_device_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_device".
            """
            ...

    @property
    def _live_ptr(self) -> int:
        ...

    def add_arm_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "arm" has changed.
        """
        ...

    def add_arrangement_clips_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "arrangement_clips" has changed.
        """
        ...

    def add_available_input_routing_channels_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "available_input_routing_channels" has changed.
        """
        ...

    def add_available_input_routing_types_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "available_input_routing_types" has changed.
        """
        ...

    def add_available_output_routing_channels_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "available_output_routing_channels" has changed.
        """
        ...

    def add_available_output_routing_types_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "available_output_routing_types" has changed.
        """
        ...

    def add_back_to_arranger_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "back_to_arranger" has changed.
        """
        ...

    def add_clip_slots_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "clip_slots" has changed.
        """
        ...

    def add_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color_index" has changed.
        """
        ...

    def add_color_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color" has changed.
        """
        ...

    def add_current_input_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "current_input_routing" has changed.
        """
        ...

    def add_current_input_sub_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "current_input_sub_routing" has changed.
        """
        ...

    def add_current_monitoring_state_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "current_monitoring_state" has changed.
        """
        ...

    def add_current_output_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "current_output_routing" has changed.
        """
        ...

    def add_current_output_sub_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "current_output_sub_routing" has changed.
        """
        ...

    def add_data_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "data" has changed.
        """
        ...

    def add_devices_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "devices" has changed.
        """
        ...

    def add_fired_slot_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "fired_slot_index" has changed.
        """
        ...

    def add_has_audio_input_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_audio_input" has changed.
        """
        ...

    def add_has_audio_output_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_audio_output" has changed.
        """
        ...

    def add_has_midi_input_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_midi_input" has changed.
        """
        ...

    def add_has_midi_output_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_midi_output" has changed.
        """
        ...

    def add_implicit_arm_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "implicit_arm" has changed.
        """
        ...

    def add_input_meter_left_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "input_meter_left" has changed.
        """
        ...

    def add_input_meter_level_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "input_meter_level" has changed.
        """
        ...

    def add_input_meter_right_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "input_meter_right" has changed.
        """
        ...

    def add_input_routing_channel_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "input_routing_channel" has changed.
        """
        ...

    def add_input_routing_type_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "input_routing_type" has changed.
        """
        ...

    def add_input_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "input_routings" has changed.
        """
        ...

    def add_input_sub_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "input_sub_routings" has changed.
        """
        ...

    def add_is_frozen_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_frozen" has changed.
        """
        ...

    def add_is_showing_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_showing_chains" has changed.
        """
        ...

    def add_mute_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mute" has changed.
        """
        ...

    def add_muted_via_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "muted_via_solo" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_output_meter_left_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "output_meter_left" has changed.
        """
        ...

    def add_output_meter_level_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "output_meter_level" has changed.
        """
        ...

    def add_output_meter_right_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "output_meter_right" has changed.
        """
        ...

    def add_output_routing_channel_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "output_routing_channel" has changed.
        """
        ...

    def add_output_routing_type_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "output_routing_type" has changed.
        """
        ...

    def add_output_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "output_routings" has changed.
        """
        ...

    def add_output_sub_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "output_sub_routings" has changed.
        """
        ...

    def add_performance_impact_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "performance_impact" has changed.
        """
        ...

    def add_playing_slot_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "playing_slot_index" has changed.
        """
        ...

    def add_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "solo" has changed.
        """
        ...

    def add_take_lanes_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "take_lanes" has changed.
        """
        ...

    @property
    def arm(self) -> bool:
        """Arm the track for recording. Not available for Main- and Send Tracks."""
        ...

    @arm.setter
    def arm(self, value: bool) -> None: ...

    def arm_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "arm".
        """
        ...

    @property
    def arrangement_clips(self) -> Vector[Clip]:
        """const access to the list of clips in arrangement viewThe list will be empty for the main, send and group tracks."""
        ...

    def arrangement_clips_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "arrangement_clips".
        """
        ...

    @property
    def available_input_routing_channels(self) -> RoutingChannelVector:
        """Return a list of source channels for input routing."""
        ...

    def available_input_routing_channels_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "available_input_routing_channels".
        """
        ...

    @property
    def available_input_routing_types(self) -> RoutingTypeVector:
        """Return a list of source types for input routing."""
        ...

    def available_input_routing_types_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "available_input_routing_types".
        """
        ...

    @property
    def available_output_routing_channels(self) -> RoutingChannelVector:
        """Return a list of destination channels for output routing."""
        ...

    def available_output_routing_channels_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "available_output_routing_channels".
        """
        ...

    @property
    def available_output_routing_types(self) -> RoutingTypeVector:
        """Return a list of destination types for output routing."""
        ...

    def available_output_routing_types_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "available_output_routing_types".
        """
        ...

    @property
    def back_to_arranger(self) -> bool:
        """Indicates if it's possible to go back to playing back the clips in the Arranger.Setting a value 0 will go back to the Arranger playback. Setting on grouptracks will go back to the Arranger on all grouped tracks."""
        ...

    @back_to_arranger.setter
    def back_to_arranger(self, value: bool) -> None: ...

    def back_to_arranger_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "back_to_arranger".
        """
        ...

    @property
    def can_be_armed(self) -> bool:
        """
        return True, if this Track has a valid arm property. Not all tracks
        can be armed (for example return Tracks or the Main Tracks).
        """
        ...

    @property
    def can_be_frozen(self) -> bool:
        """return True, if this Track can be frozen."""
        ...

    @property
    def can_show_chains(self) -> bool:
        """return True, if this Track contains a rack instrument device that is capable of showing its chains in session view."""
        ...

    @property
    def canonical_parent(self) -> Song:
        """Get the canonical parent of the track."""
        ...

    @property
    def clip_slots(self) -> Vector[ClipSlot]:
        """
        const access to the list of clipslots (see class AClipSlot) for this track.
        The list will be empty for the main and sendtracks.
        """
        ...

    def clip_slots_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "clip_slots".
        """
        ...

    @property
    def color(self) -> int:
        """Get/set access to the color of the Track (RGB)."""
        ...

    @color.setter
    def color(self, value: int) -> None: ...

    def color_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color".
        """
        ...

    @property
    def color_index(self) -> int:
        """Get/Set access to the color index of the track. Can be None for no color."""
        ...

    @color_index.setter
    def color_index(self, value: int) -> None: ...

    def color_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color_index".
        """
        ...

    def create_audio_clip(self, file_path: str | None, position: float | None, /) -> Clip:
        """
        Creates an audio clip referencing the file at the given path and inserts it into the arrangement at the specified time.
        Throws an error when called on a non-audio or a frozen track, when the specified time is outside the [0., 1576800.] range, when the track is currently being recorded into, or when the path doesn't point to a valid audio file.
        """
        ...

    def create_midi_clip(self, start_time: float | None, length: float | None, /) -> Clip:
        """
        Creates an empty MIDI clip and inserts it into the arrangement at the specified time.
        Throws an error when called on a non-MIDI track or a frozen track, when the specified time is outside the [0., 1576800.] range, or when the track is currently being recorded into.
        """
        ...

    def create_take_lane(self) -> TakeLane:
        """Create a new TakeLane for this track."""
        ...

    @property
    def current_input_routing(self) -> str:
        """
        Get/Set the name of the current active input routing.
        When setting a new routing, the new routing must be one of the available ones.
        """
        ...

    @current_input_routing.setter
    def current_input_routing(self, value: str) -> None: ...

    def current_input_routing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "current_input_routing".
        """
        ...

    @property
    def current_input_sub_routing(self) -> str:
        """
        Get/Set the current active input sub routing.
        When setting a new routing, the new routing must be one of the available ones.
        """
        ...

    @current_input_sub_routing.setter
    def current_input_sub_routing(self, value: str) -> None: ...

    def current_input_sub_routing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "current_input_sub_routing".
        """
        ...

    @property
    def current_monitoring_state(self) -> int:
        """Get/Set the track's current monitoring state."""
        ...

    @current_monitoring_state.setter
    def current_monitoring_state(self, value: int) -> None: ...

    def current_monitoring_state_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "current_monitoring_state".
        """
        ...

    @property
    def current_output_routing(self) -> str:
        """
        Get/Set the current active output routing.
        When setting a new routing, the new routing must be one of the available ones.
        """
        ...

    @current_output_routing.setter
    def current_output_routing(self, value: str) -> None: ...

    def current_output_routing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "current_output_routing".
        """
        ...

    @property
    def current_output_sub_routing(self) -> str:
        """
        Get/Set the current active output sub routing.
        When setting a new routing, the new routing must be one of the available ones.
        """
        ...

    @current_output_sub_routing.setter
    def current_output_sub_routing(self, value: str) -> None: ...

    def current_output_sub_routing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "current_output_sub_routing".
        """
        ...

    def data_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "data".
        """
        ...

    def delete_clip(self, slot: Clip | None, /) -> None:
        """Delete the given clip. Raises a runtime error when the clip belongs to another track."""
        ...

    def delete_device(self, device: int | None, /) -> None:
        """Delete a device identified by the index in the 'devices' list."""
        ...

    @property
    def devices(self) -> Vector[Device]:
        """
        Return const access to all available Devices that are present in the Tracks
        Devicechain. This tuple will also include the 'mixer_device' that every Track
        always has.
        """
        ...

    def devices_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "devices".
        """
        ...

    def duplicate_clip_slot(self, index: int | None, /) -> int:
        """
        Duplicate a clip and put it into the next free slot and return the index
        of the destination slot. A new scene is created if no free slot is
        available. If creating the new scene would exceed the limitations,
        a runtime error is raised.
        """
        ...

    def duplicate_clip_to_arrangement(self, clip: Clip | None, destination_time: float | None, /) -> Clip:
        """
        Duplicate the given clip into the arrangement of this track at the provided
        destination time and return it. When the type of the clip and the type of the
        track are incompatible, a runtime error is raised.
        """
        ...

    def duplicate_device(self, index: int | None, /) -> None:
        """Duplicate a device at a given index in the 'devices' list."""
        ...

    @property
    def fired_slot_index(self) -> int:
        """
        const access to the index of the fired (and thus blinking) clipslot in this track.
        This index is -1 if no slot is fired and -2 if the track's stop button has been fired.
        """
        ...

    def fired_slot_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "fired_slot_index".
        """
        ...

    @property
    def fold_state(self) -> bool:
        """Get/Set whether the track is folded or not. Only available if is_foldable is True."""
        ...

    @fold_state.setter
    def fold_state(self, value: bool) -> None: ...

    def get_data(self, key: str | None, default_value: Any | None, /) -> Any:
        """Get data for the given key, that was previously stored using set_data."""
        ...

    @property
    def group_track(self) -> Track:
        """return the group track if is_grouped."""
        ...

    @property
    def has_audio_input(self) -> bool:
        """
        return True, if this Track can be feed with an Audio signal. This is
        true for all Audio Tracks.
        """
        ...

    def has_audio_input_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_audio_input".
        """
        ...

    @property
    def has_audio_output(self) -> bool:
        """
        return True, if this Track sends out an Audio signal. This is
        true for all Audio Tracks, and MIDI tracks with an Instrument.
        """
        ...

    def has_audio_output_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_audio_output".
        """
        ...

    @property
    def has_midi_input(self) -> bool:
        """
        return True, if this Track can be feed with an Audio signal. This is
        true for all MIDI Tracks.
        """
        ...

    def has_midi_input_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_midi_input".
        """
        ...

    @property
    def has_midi_output(self) -> bool:
        """
        return True, if this Track sends out MIDI events. This is
        true for all MIDI Tracks with no Instruments.
        """
        ...

    def has_midi_output_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_midi_output".
        """
        ...

    @property
    def implicit_arm(self) -> bool:
        """Arm the track for recording. When The track is implicitly armed, it showsin a weaker color in the live GUI and is not saved in the set."""
        ...

    @implicit_arm.setter
    def implicit_arm(self, value: bool) -> None: ...

    def implicit_arm_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "implicit_arm".
        """
        ...

    @property
    def input_meter_left(self) -> float:
        """Momentary value of left input channel meter, 0.0 to 1.0. For Audio Tracks only."""
        ...

    def input_meter_left_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "input_meter_left".
        """
        ...

    @property
    def input_meter_level(self) -> float:
        """
        Return the MIDI or Audio meter value of the Tracks input, depending on the
        type of the Track input. Meter values (MIDI or Audio) are always scaled
        from 0.0 to 1.0.
        """
        ...

    def input_meter_level_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "input_meter_level".
        """
        ...

    @property
    def input_meter_right(self) -> float:
        """Momentary value of right input channel meter, 0.0 to 1.0. For Audio Tracks only."""
        ...

    def input_meter_right_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "input_meter_right".
        """
        ...

    @property
    def input_routing_channel(self) -> RoutingChannel:
        """
        Get and set the current source channel for input routing.
        Raises ValueError if the type isn't one of the current values in
        available_input_routing_channels.
        """
        ...

    @input_routing_channel.setter
    def input_routing_channel(self, value: RoutingChannel) -> None: ...

    def input_routing_channel_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "input_routing_channel".
        """
        ...

    @property
    def input_routing_type(self) -> RoutingType:
        """
        Get and set the current source type for input routing.
        Raises ValueError if the type isn't one of the current values in
        available_input_routing_types.
        """
        ...

    @input_routing_type.setter
    def input_routing_type(self, value: RoutingType) -> None: ...

    def input_routing_type_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "input_routing_type".
        """
        ...

    @property
    def input_routings(self) -> StringVector:
        """Const access to the list of available input routings."""
        ...

    def input_routings_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "input_routings".
        """
        ...

    @property
    def input_sub_routings(self) -> StringVector:
        """Return a list of all available input sub routings."""
        ...

    def input_sub_routings_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "input_sub_routings".
        """
        ...

    def insert_device(self, device_name: str | None, device_index: int = -1, /) -> Device:
        """Add a device at a given index in the 'devices' list. At end if -1."""
        ...

    @property
    def is_foldable(self) -> bool:
        """return True if the track can be (un)folded to hide/reveal contained tracks."""
        ...

    @property
    def is_frozen(self) -> bool:
        """return True if this Track is currently frozen. No changes should be applied to the track's devices or clips while it is frozen."""
        ...

    def is_frozen_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_frozen".
        """
        ...

    @property
    def is_grouped(self) -> bool:
        """return True if this Track is current part of a group track."""
        ...

    @property
    def is_part_of_selection(self) -> bool:
        """return False if the track is not selected."""
        ...

    @property
    def is_showing_chains(self) -> bool:
        """Get/Set whether a track with a rack device is showing its chains in session view."""
        ...

    @is_showing_chains.setter
    def is_showing_chains(self, value: bool) -> None: ...

    def is_showing_chains_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_showing_chains".
        """
        ...

    @property
    def is_visible(self) -> bool:
        """return False if the track is hidden within a folded group track."""
        ...

    def jump_in_running_session_clip(self, beats: float | None, /) -> None:
        """
        Jump forward or backward in the currently running Sessionclip (if any)
        by the specified relative amount in beats. Does nothing if no Session Clip
        is currently running.
        """
        ...

    @property
    def mixer_device(self) -> MixerDevice:
        """
        Return access to the special Device that every Track has: This Device contains
        the Volume, Pan, Sendamounts, and Crossfade assignment parameters.
        """
        ...

    class monitoring_states(int):
        IN: int = 0
        AUTO: int = 1
        OFF: int = 2

    @property
    def mute(self) -> bool:
        """Mute/unmute the track."""
        ...

    @mute.setter
    def mute(self, value: bool) -> None: ...

    def mute_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mute".
        """
        ...

    @property
    def muted_via_solo(self) -> bool:
        """Returns true if the track is muted because another track is soloed."""
        ...

    def muted_via_solo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "muted_via_solo".
        """
        ...

    @property
    def name(self) -> str:
        """Read/write access to the name of the Track, as visible in the track header."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    @property
    def output_meter_left(self) -> float:
        """
        Momentary value of left output channel meter, 0.0 to 1.0.
        For tracks with audio output only.
        """
        ...

    def output_meter_left_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "output_meter_left".
        """
        ...

    @property
    def output_meter_level(self) -> float:
        """
        Return the MIDI or Audio meter value of the Track output (behind the
        mixer_device), depending on the type of the Track input, this can be a MIDI
        or Audio meter. Meter values (MIDI or Audio) are always scaled from 0.0 to 1.0.
        """
        ...

    def output_meter_level_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "output_meter_level".
        """
        ...

    @property
    def output_meter_right(self) -> float:
        """
        Momentary value of right output channel meter, 0.0 to 1.0.
        For tracks with audio output only.
        """
        ...

    def output_meter_right_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "output_meter_right".
        """
        ...

    @property
    def output_routing_channel(self) -> RoutingChannel:
        """
        Get and set the current destination channel for output routing.
        Raises ValueError if the channel isn't one of the current values in
        available_output_routing_channels.
        """
        ...

    @output_routing_channel.setter
    def output_routing_channel(self, value: RoutingChannel) -> None: ...

    def output_routing_channel_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "output_routing_channel".
        """
        ...

    @property
    def output_routing_type(self) -> RoutingType:
        """
        Get and set the current destination type for output routing.
        Raises ValueError if the type isn't one of the current values in
        available_output_routing_types.
        """
        ...

    @output_routing_type.setter
    def output_routing_type(self, value: RoutingType) -> None: ...

    def output_routing_type_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "output_routing_type".
        """
        ...

    @property
    def output_routings(self) -> StringVector:
        """Const access to the list of all available output routings."""
        ...

    def output_routings_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "output_routings".
        """
        ...

    @property
    def output_sub_routings(self) -> StringVector:
        """Return a list of all available output sub routings."""
        ...

    def output_sub_routings_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "output_sub_routings".
        """
        ...

    @property
    def performance_impact(self) -> float:
        """Reports the performance impact of this track."""
        ...

    def performance_impact_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "performance_impact".
        """
        ...

    @property
    def playing_slot_index(self) -> int:
        """
        const access to the index of the currently playing clip in the track.
        Will be -1 when no clip is playing.
        """
        ...

    def playing_slot_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "playing_slot_index".
        """
        ...

    def remove_arm_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "arm".
        """
        ...

    def remove_arrangement_clips_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "arrangement_clips".
        """
        ...

    def remove_available_input_routing_channels_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "available_input_routing_channels".
        """
        ...

    def remove_available_input_routing_types_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "available_input_routing_types".
        """
        ...

    def remove_available_output_routing_channels_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "available_output_routing_channels".
        """
        ...

    def remove_available_output_routing_types_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "available_output_routing_types".
        """
        ...

    def remove_back_to_arranger_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "back_to_arranger".
        """
        ...

    def remove_clip_slots_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "clip_slots".
        """
        ...

    def remove_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color_index".
        """
        ...

    def remove_color_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color".
        """
        ...

    def remove_current_input_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "current_input_routing".
        """
        ...

    def remove_current_input_sub_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "current_input_sub_routing".
        """
        ...

    def remove_current_monitoring_state_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "current_monitoring_state".
        """
        ...

    def remove_current_output_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "current_output_routing".
        """
        ...

    def remove_current_output_sub_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "current_output_sub_routing".
        """
        ...

    def remove_data_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "data".
        """
        ...

    def remove_devices_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "devices".
        """
        ...

    def remove_fired_slot_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "fired_slot_index".
        """
        ...

    def remove_has_audio_input_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_audio_input".
        """
        ...

    def remove_has_audio_output_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_audio_output".
        """
        ...

    def remove_has_midi_input_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_midi_input".
        """
        ...

    def remove_has_midi_output_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_midi_output".
        """
        ...

    def remove_implicit_arm_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "implicit_arm".
        """
        ...

    def remove_input_meter_left_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "input_meter_left".
        """
        ...

    def remove_input_meter_level_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "input_meter_level".
        """
        ...

    def remove_input_meter_right_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "input_meter_right".
        """
        ...

    def remove_input_routing_channel_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "input_routing_channel".
        """
        ...

    def remove_input_routing_type_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "input_routing_type".
        """
        ...

    def remove_input_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "input_routings".
        """
        ...

    def remove_input_sub_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "input_sub_routings".
        """
        ...

    def remove_is_frozen_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_frozen".
        """
        ...

    def remove_is_showing_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_showing_chains".
        """
        ...

    def remove_mute_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mute".
        """
        ...

    def remove_muted_via_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "muted_via_solo".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_output_meter_left_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "output_meter_left".
        """
        ...

    def remove_output_meter_level_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "output_meter_level".
        """
        ...

    def remove_output_meter_right_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "output_meter_right".
        """
        ...

    def remove_output_routing_channel_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "output_routing_channel".
        """
        ...

    def remove_output_routing_type_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "output_routing_type".
        """
        ...

    def remove_output_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "output_routings".
        """
        ...

    def remove_output_sub_routings_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "output_sub_routings".
        """
        ...

    def remove_performance_impact_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "performance_impact".
        """
        ...

    def remove_playing_slot_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "playing_slot_index".
        """
        ...

    def remove_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "solo".
        """
        ...

    def remove_take_lanes_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "take_lanes".
        """
        ...

    def set_data(self, key: str | None, value: Any | None, /) -> None:
        """Store data for the given key in this object. The data is persistent and will be restored when loading the Live Set."""
        ...

    @property
    def solo(self) -> bool:
        """
        Get/Set the solo status of the track. Note that this will not disable the
        solo state of any other track. If you want exclusive solo, you have to
        disable the solo state of the other Tracks manually.
        """
        ...

    @solo.setter
    def solo(self, value: bool) -> None: ...

    def solo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "solo".
        """
        ...

    def stop_all_clips(self, quantized: bool = True, /) -> None:
        """Stop running and triggered clip and slots on this track."""
        ...

    @property
    def take_lanes(self) -> Vector[TakeLane]:
        """returns the take lanes."""
        ...

    def take_lanes_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "take_lanes".
        """
        ...

    @property
    def view(self) -> View:
        """Representing the view aspects of a Track."""
        ...

class DeviceContainer(LomObject):
    """This class is a common super class of Track and Chain"""

    @property
    def _live_ptr(self) -> int:
        ...

class DeviceInsertMode(int):
    default: int = 0
    selected_left: int = 1
    selected_right: int = 2
    count: int = 3

class RoutingChannel:
    """This class represents a routing channel."""

    @property
    def display_name(self) -> str:
        """Display name of routing channel."""
        ...

    @property
    def layout(self) -> RoutingChannelLayout:
        """The routing channel's Layout, e.g., mono or stereo."""
        ...

class RoutingChannelLayout(int):
    midi: int = 0
    mono: int = 1
    stereo: int = 2

class RoutingChannelVector(Vector[RoutingChannel]):
    """A container for returning routing channels from Live."""

    def append(self, value: RoutingChannel | None, /) -> None:
        ...

    def extend(self, values: Iterable[RoutingChannel] | None, /) -> None:
        ...

class RoutingType:
    """This class represents a routing type."""

    @property
    def attached_object(self) -> Track:
        """Live object associated with the routing type."""
        ...

    @property
    def category(self) -> int:
        """Category of the routing type."""
        ...

    @property
    def display_name(self) -> str:
        """Display name of routing type."""
        ...

class RoutingTypeCategory(int):
    external: int = 0
    rewire: int = 1
    resampling: int = 2
    master: int = 3
    track: int = 4
    parent_group_track: int = 5
    none: int = 6
    invalid: int = 7

class RoutingTypeVector(Vector[RoutingType]):
    """A container for returning routing types from Live."""

    def append(self, value: RoutingType | None, /) -> None:
        ...

    def extend(self, values: Iterable[RoutingType] | None, /) -> None:
        ...

__all__ = ['Track', 'DeviceContainer', 'DeviceInsertMode', 'RoutingChannel', 'RoutingChannelLayout', 'RoutingChannelVector', 'RoutingType', 'RoutingTypeCategory', 'RoutingTypeVector']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Chain import Chain
    from Live.ChainMixerDevice import ChainMixerDevice
    from Live.RackDevice import RackDevice
    from Live.Track import Track



class DrumChain(Chain):
    """This class represents a drum group device chain in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_choke_group_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "choke_group" has changed.
        """
        ...

    def add_in_note_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "in_note" has changed.
        """
        ...

    def add_out_note_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "out_note" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> RackDevice:
        """Get the canonical parent of the chain."""
        ...

    @property
    def choke_group(self) -> int:
        """Access to the chain's choke group setting."""
        ...

    @choke_group.setter
    def choke_group(self, value: int) -> None: ...

    def choke_group_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "choke_group".
        """
        ...

    @property
    def color(self) -> int:
        """Access the color index of the Chain."""
        ...

    @color.setter
    def color(self, value: int) -> None: ...

    @property
    def color_index(self) -> int:
        """Access the color index of the Chain."""
        ...

    @color_index.setter
    def color_index(self, value: int) -> None: ...

    @property
    def devices(self) -> Vector[RackDevice]:
        """Return const access to all available Devices that are present in the chains"""
        ...

    @property
    def has_audio_input(self) -> bool:
        """
        return True, if this Chain can be feed with an Audio signal. This is
        true for all Audio Chains.
        """
        ...

    @property
    def has_audio_output(self) -> bool:
        """
        return True, if this Chain sends out an Audio signal. This is
        true for all Audio Chains, and MIDI chains with an Instrument.
        """
        ...

    @property
    def has_midi_input(self) -> bool:
        """
        return True, if this Chain can be feed with an Audio signal. This is
        true for all MIDI Chains.
        """
        ...

    @property
    def has_midi_output(self) -> bool:
        """
        return True, if this Chain sends out MIDI events. This is
        true for all MIDI Chains with no Instruments.
        """
        ...

    @property
    def in_note(self) -> int:
        """Access to the incoming MIDI note that will trigger this chain."""
        ...

    @in_note.setter
    def in_note(self, value: int) -> None: ...

    def in_note_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "in_note".
        """
        ...

    @property
    def is_auto_colored(self) -> bool:
        """
        Get/set access to the auto color flag of the Chain.
        If True, the Chain will always have the same color as the containing
        Track or Chain.
        """
        ...

    @is_auto_colored.setter
    def is_auto_colored(self, value: bool) -> None: ...

    @property
    def mixer_device(self) -> ChainMixerDevice:
        """
        Return access to the mixer device that holds the chain's mixer parameters:
        the Volume, Pan, and Sendamounts.
        """
        ...

    @property
    def mute(self) -> bool:
        """Mute/unmute the chain."""
        ...

    @mute.setter
    def mute(self, value: bool) -> None: ...

    @property
    def muted_via_solo(self) -> bool:
        """
        Return const access to whether this chain is muted due to some other chain
        being soloed.
        """
        ...

    @property
    def name(self) -> str:
        """Read/write access to the name of the Chain, as visible in the track header."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def out_note(self) -> int:
        """Access to the MIDI note sent to the devices in the chain."""
        ...

    @out_note.setter
    def out_note(self, value: int) -> None: ...

    def out_note_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "out_note".
        """
        ...

    def remove_choke_group_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "choke_group".
        """
        ...

    def remove_in_note_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "in_note".
        """
        ...

    def remove_out_note_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "out_note".
        """
        ...

    @property
    def solo(self) -> bool:
        """
        Get/Set the solo status of the chain. Note that this will not disable the
        solo state of any other Chain in the same rack. If you want exclusive solo,
        you have to disable the solo state of the other Chains manually.
        """
        ...

    @solo.setter
    def solo(self, value: bool) -> None: ...

__all__ = ['DrumChain']
